function configure_labjack(L)

% Apply configuration specified in LJ-object
%
% Author: DJ van Gerwen
% Created: 2016 june 5


switch L.device_type
    case 'U3'
        % Reset pin configuration to factory default
        chanType = LabJack.LabJackUD.CHANNEL.LOCALID.GetType;
        chanObj = System.Enum.ToObject(chanType, 0); % channel = 0
        L.UDobject.ePut(L.lj_handle, LabJack.LabJackUD.IO.PIN_CONFIGURATION_RESET, ...
            chanObj, 0, 0);
        % NOTE: the system.enum does not have to be used: can just use integer.
        
        % Configure FIO ports as analog (for the U3HV, FIO0 to FIO3 are fixed as analog inputs, so this operation is ignored)
        FIO_config = repmat('0',1,16); % Initialize FIO configuration bits for all 16 ports (rightmost bit corresponds with FIO0 (or AIN0 on the U3HV))
        FIO_config(end-L.AIN_list) = '1'; % Set bits for analog ports (counting from right to left)
        L.UDobject.ePut(L.lj_handle, LabJack.LabJackUD.IO.PUT_ANALOG_ENABLE_PORT, ...
            0, bin2dec(FIO_config), int32(16));
        % NOTE: The last three arguments are: starting channel (corresponds to
        % rightmost bit), decimal representation of the configuration bits (right to left),
        % and number of configuration bits to update (max 16 bits, for all 16 channels).
        % For example, to configure ports FIO4-FIO5 as analog, you could use
        % either ...,4,b11=3,2), or ...,0,b0000000000110000=48,16)
        
        L.AIN_range_V = 10 * ones(size(L.AIN_list)); % For U3HV
        
        
    case 'UE9'
        
        % Configure resolution for all analog inputs (12-bit allows highest sample rate)
        LJ_chAIN_RESOLUTION = L.UDobject.StringToConstant('LJ_chAIN_RESOLUTION');
        L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.PUT_CONFIG, ...
            LJ_chAIN_RESOLUTION, L.resolution_bits, 0, 0);
        
        % TODO: Create observed range property, similar to daq-toolbox
        
        % Configure analog inputs for 5V range
        LJ_rgBIP5V = L.UDobject.StringToConstant('LJ_rgBIP5V');
        for AINi = L.AIN_list
            L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.PUT_AIN_RANGE, ...
                AINi, LJ_rgBIP5V, 0, 0);
        end
        L.AIN_range_V = 5 * ones(size(L.AIN_list));
        
    case 'T7'
        % Stream Configuration
        %aScanListNames = NET.createArray('System.String', L.number_of_AIN); %Scan list names to stream.
        %aScanListNames(1) = 'AIN0';
        %aScanListNames(2) = 'AIN1';
        %for i = 1:L.number_of_AIN
        %    aScanListNames{i} = sprintf('AIN%u',L.AIN_list(i));
        %end
        %aScanList = zeros(1,L.number_of_AIN);%NET.createArray('System.Int32', L.number_of_AIN); %Scan list addresses to stream.
        %aTypes =  zeros(1,L.number_of_AIN);%NET.createArray('System.Int32', L.number_of_AIN); %Dummy array for aTypes parameter
        %LabJack.LJM.NamesToAddresses(L.number_of_AIN, aScanListNames, aScanList, aTypes);
        
        %scanRate = double(1000); %Scans per second
        %scansPerRead = int32(scanRate/2);
        %Stream reads will be stored in aData. Needs to be at least
        %NumAddresses*ScansPerRead in size.
        %aData = zeros(1,L.number_of_AIN*scansPerRead);%NET.createArray('System.Double', L.number_of_AIN*scansPerRead);
        
        L.AIN_range_V = 10 * ones(size(L.AIN_list));
                
        %Configure the negative channels for single ended readings.
        %aNames = NET.createArray('System.String', L.number_of_AIN);
        %aValues = NET.createArray('System.Double', L.number_of_AIN);
        for i = 1:3:3*L.number_of_AIN,
            aNames{i}   = sprintf( 'AIN%u_NEGATIVE_CH', L.AIN_list(i));
            aNames{i+1} = sprintf( 'AIN%u_RANGE', L.AIN_list(i));
            aNames{i+2} = sprintf( 'AIN%u_RESOLUTION_INDEX', L.AIN_list(i));
            aValues(i)  = L.Mconstants.GND; % Set ground as reference for specified channel
            aValues(i+1) = L.AIN_range_V; % Set range (0.1, 1, or 10) [V]
            aValues(i+2) = 1; % Set resolution index (default=0) [-] Effective resolution for 10V range: 1=16bit, ..., 8=19.1bit, (T7-pro can go up to 12=21.8bit) [https://labjack.com/support/datasheets/t7/appendix-a-3-1]
        end
        LabJack.LJM.eWriteNames(L.lj_handle, L.number_of_AIN, aNames, aValues, 0);
        
end

if ismember(L.device_type, {'U3', 'UE9'})
    
    % Set scan rate (scan rate = sample rate per channel)
    L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.PUT_CONFIG, ...
        LabJack.LabJackUD.CHANNEL.STREAM_SCAN_FREQUENCY, L.scan_rate_requested_Hz, 0, 0);
    
    % Set buffer size (scan_rate_requested_Hz * num_channels * buffer_time_s)
    L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.PUT_CONFIG, ...
        LabJack.LabJackUD.CHANNEL.STREAM_BUFFER_SIZE,L.buffer_size, 0, 0);
    
    % Configure stream to retrieve whatever data is available without waiting
    L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.PUT_CONFIG, ...
        LabJack.LabJackUD.CHANNEL.STREAM_WAIT_MODE, 1, 0, 0);
    
    % Clear stream channels
    L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.CLEAR_STREAM_CHANNELS, ...
        0, 0, 0, 0);
    
    % Configure analog inputs for streaming
    for AINi = L.AIN_list
        L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.ADD_STREAM_CHANNEL, ...
            AINi, 0, 0, 0);
    end
    
    if ~strcmp(L.device_type, 'U3')
        % Configure digital inputs for streaming
        % NOTE: channel 193 reads 16 bit value from EIO and FIO (FIO 0-7 is lower 8 bits)
        % NOTE: The required FIO channels need to be configured for input for
        % streaming to work. This is done by the set.DIN_list method.
        if ~isempty(L.DIN_list)
            DIN_channel = 193;
            L.UDobject.AddRequest(L.lj_handle, LabJack.LabJackUD.IO.ADD_STREAM_CHANNEL, ...
                DIN_channel, 0, 0, 0);
        end
    end
    
    % NOTE: The individual bits can be extracted from the stream using an
    % approach as below (see soloist.saveCapture method for example)
    % all_bits = dec2bin(stream_data(iochannel:number_of_channels:end));
    % last_two_bits = bin2dec(all_bits(:,end-2:end-1));
     
    % Execute the list of requests.
    L.UDobject.GoOne(L.lj_handle);
    
    % Get all the results just to check for errors.
    L.UDobject.GetFirstResult(L.lj_handle, LabJack.LabJackUD.IO, LabJack.LabJackUD.CHANNEL, 0, 0, 0);
    finished = 0;
    while ~finished
        try
            L.UDobject.GetNextResult(L.lj_handle, LabJack.LabJackUD.IO, LabJack.LabJackUD.CHANNEL, 0, 0, 0);
        catch errmsg
            if isa(errmsg, 'NET.NetException')
                eNet = errmsg.ExceptionObject;
                if isa(eNet, 'LabJack.LabJackUD.LabJackUDException')
                    if eNet.LJUDError == LabJack.LabJackUD.LJUDERROR.NO_MORE_DATA_AVAILABLE
                        finished = 1;
                    end
                end
            end
            if ~finished
                throw(errmsg) % Report errors other than NO_MORE_DATA_AVAILABLE
            end
        end
    end
    
elseif strcmp(L.device_type,'T7')
    
    % T7 stream configuration is performed in the start stream function (LabJack.LJM.eStreamStart)...
    
end

end % Configure