function PillowExe()
%% This m-file calls GUI and data acquisition for pillow force measurements
% Author: dr.ir. Arjo J. Loeve, ir. Nick van de berg
% E-mail: a.j.loeve@tudelft.nl
% Date: 2016AUG15-17.45
% LabJack readout functions and class: dr.ir. Dennis J. van Gerwen
% Thanks for GUI example codes: ir. Nick van de Berg
%close all; clear all; clc;
global measflag; % Measurement actively running
global readflag; % Measurement terminated or completed
measflag = 0;
readflag = 0;

%% Initialize GUI
% Make structure h
h.Made = 1;

% Define screen size
resH = 1280; % Horizontal screen resolution of aspect ratio in pixels
resV = 1024; % Vertical screen resolution of aspect in pixels

% Define accepted barcode size
h.intLBarcode = 4;
% Make trial run variable to be entered in Subjnr when no saves must be made
h.Trialstr = '000';

% Define file naming and column header variables
strFileSubjPre = {[pwd '\DATA\LWLPSubj-']};
strFileAll = {[pwd '\DATA\LWLPillowALL-Overview.mat']};
celColHeadersSummative= {'Nr of Runs','SaveDate','SaveTime','Mean RunClock [s]',...
                        'Mean F_mean [N]','Mean F_max [N]',...
                        'Max F_mean [N]','Max F_max [N]',...
                        'Mean t_run [s]','Mean t_smo [s]','Mean t_val [s]',...
                        'Percentage Completed [%]','Mean NrOfPauses'};
celColHeadersOverview = {'SubjNr','SaveDate','SaveTime','RunClock [s]',...
                        'F_mean [N]','F_max [N]',...
                        't_run [s]','t_smothering [s]', 't_valid [s]',...
                        'Completed','NrOfPauses','Notes'};
celColHeadersData = {'Runtime [s]', 'Sensor Signal [V]',...
                    'Push Force [N]','Paused [0=no/1=yes]'};

% Create main figure
figGUI = figure('Tag','figGUI','Position',[0 0 resH resV],...
    'Name','Pillow Force Measurements','Color',[0 0 0],'MenuBar','none',...
    'Toolbar','none','NumberTitle','off','Visible','off');

% Make measurement constants and provide CALLIBRATION factors
h.SampleT = 1/20; % Sampletime in s (1000ms/Frequency)
h.SmotherF = 50; % Threshold force required to suffocate [N]
h.SmotherT = 45*h.SmotherF; % F[N]*T[s] required above threshold force to suffocate
h.SmotherProg = '0'; % Indicator showing progress of smothering
h.VtoF = [0 210]; % [a b] Callibration factors Volts to Newtons: F=a+b*V

% Define figure properties for data plotting
h.MaxExpF = 1300; % Maximum expected force, hence Ylim
h.MaxVisT = 10; % Maximum time frame to be shown in plot
h.PlotLength = 1+h.MaxVisT/h.SampleT; % Nr of samples plotted maximally in figure

% Main function creating GUI components
[h] = PillowGUI(figGUI,h);

% Store some values to avoid double definitions
h.strNextClean = h.Next.String; % This is the clean caption as set in GUI
h.strNextSaving = 'SAVING data...'; % During saving
h.strNextSave = 'SAVE & Reset'; % Only when measured and fully stopped
h.strNextReset = 'RESET'; % Only before measurement started
h.strNextBusy = 'Measuring...'; % During measuring and pauses

% Callback definitions
set(h.Start,    'Callback',{@callStart},	'Tag','butStart');
set(h.Stop,     'Callback',{@callStop},     'Tag','butStop');
set(h.Next,     'Callback',{@callNext},     'Tag','butNext');
set(h.edtSubj,  'Callback',{@callSubj},     'Tag','edtSubj');
set(h.edtNotes,  'Callback',{@callNotes},   'Tag','edtNotes');
set(figGUI,     'DeleteFcn',{@callCloseGUI},'Tag','figGUI');

% Make data variables
h.Subjnr = '';
h.DataT = []; % Time stamps
h.DataT_run= 0; % Total time of test run, INCLUDING pauses, calculated
h.DataT_smo= 0; % Total excluding pauses, calculated
h.DataT_val = 0; % Total time above threshold, calculated
h.DataT_clock = []; % Total run time clocked (first to last sample)
h.DataV = []; % Voltages
h.DataF = []; % Forces
h.DataP = []; % Paused 0=no 1=yes
h.DataNotes = {}; % Notes
% h.DataP = []; % Contains start and end of all pauses...TO BE DONE
% Make status variables
h.Scanned = 0;
h.Completed = 0;
h.Started = 0;
h.intPauses = 0;
h.Terminated = 0;

% Create LabJack object
h.LJ=LJ;
% Set LabJack settings
h.LJ.AIN_list=0;                % Analogue input
h.LJ.scan_rate_requested_Hz=1000/h.SampleT;% Sampling rate [Hz] in LabJack

% Update user data
set(figGUI,'UserData',h); % Handles 'h' stored in figure UserData

% Make GUI visible
set(figGUI,'Visible','on');
uicontrol(h.edtSubj);

    function [] = callSubj(~,~)
    % After entering Subjnr in editbox, check if number is correct, set
    % variable Subjnr, warn user that file already exists if so and
    % enable appropriate controls

        % Obtain user data from main figure
        h = get(figGUI,'UserData');
        % Assure that text box contents will not be changed while handled
        h.edtSubj.Enable='off';
        h.Scanned = 1;
        h.Subjnr = num2str(h.edtSubj.String); % Assure that it's a string!
        % Check input validity: Testrun code or string of barcode length.
        if strcmp(h.Subjnr,h.Trialstr) || size(h.Subjnr,2)==h.intLBarcode
            % Check if Subjnr already has file. If so, warn and advise user.
            h.strFileSubj = strcat(strFileSubjPre,h.Subjnr,'.mat');
            if exist(h.strFileSubj{1},'file')
                strReply1 = '1) Re-enter Subject Number';
                strReply2 = '2) Overwrite previous file';
                % Notify user and clear or continue with overwrite
                msgReply = questdlg(['Datafile for Subject Number'...
                     ' already exists. Please choose how to continue.'...
                     ' If you choose to overwrite, the previous data'...
                     ' will be lost once you save the new measurement.'...
                     ' To keep both, change the name of the existing'...
                     ' file and then try again.'],...
                     'Datafile already exists!',...
                     strReply1, strReply2, strReply1); 
                if strcmp(msgReply,strReply2)
                   % Enable Start button, Notes field and Next button
                    h.Start.Enable = 'on';
                    h.edtSubj.Enable = 'off';
                    h.edtNotes.Enable = 'on';
                    h.Next.String = h.strNextReset; 
                    h.Next.Enable = 'on';
                    uicontrol(h.edtNotes);
                else % In all other cases, reset
                    h.edtSubj.String = ''; % Clear editbox
                    h.Scanned = 0;
                    h.Start.Enable = 'off';
                    h.edtNotes.Enable = 'off';
                    h.edtSubj.Enable = 'on'; % Allow new entry
                    uicontrol(h.edtSubj);
                end                
            else
                % Enable Start button, Notes field and Next button
                h.Start.Enable = 'on';
                h.edtSubj.Enable = 'off';
                h.edtNotes.Enable = 'on';
                h.Next.String = h.strNextReset;
                h.Next.Enable = 'on';
                uicontrol(h.edtNotes);
            end
        else
            % Notify user of invalid entry and clear
            strWarning=strcat('The entered Subject Number is invalid.',...
                ' It should either be < ', h.Trialstr,...
                ' > for unregistered test runs or a valid barcode',...
                ' for registered measurements.',...
                ' Field will now be cleared.');
            warndlg(strWarning,'Invalid Subject Number',...
                'modal');
            h.edtSubj.String = ''; % Clear editbox
            h.Scanned = 0;
            h.edtSubj.Enable = 'on'; % Allow new entry
            uicontrol(h.edtSubj);
        end
        % Update user data main figure
        set(figGUI,'UserData',h);
    end

     function [] = callNotes(~,~)
        % Store Notes in h whenever it has been changed
        h = get(figGUI,'UserData');
        h.DataNotes = h.edtNotes.String;
        set(figGUI,'UserData',h);
     end

    function [] = callStart(~,~)
            % Obtain user data from main figure
            h = get(figGUI,'UserData');
            if readflag % Measurement was started
                if ~measflag % Continue
                    % Measurement was started but paused, so continue it
                    h.Start.String = 'Pause';
                    h.Start.BackgroundColor = [0 114/255 189/255];
                    h.lblMes.String = h.MesKeep;
                    h.lblMes.BackgroundColor = h.cssCBackMesKeep;
                    h.Stop.Enable = 'off';
                    measflag = 1;
                else % Pause
                    % Measurement was running, so pause it
                    measflag = 0;
                    h.Start.String = 'Continue';
                    h.Start.BackgroundColor = h.cssCBackMesPaused;
                    h.lblMes.String = h.MesPaused;
                    h.lblMes.BackgroundColor = h.cssCBackMesPaused;
                    h.Stop.Enable='on';
                end
                    % Updating user data main figure would be nice
                    % but that resets the Data fields in h... solvable?
                    % So here we're not using: set(figGUI,'UserData',h);
            else % Start
                % There was no running measurement, so start it
                h.Start.String='Pause';
                h.Start.BackgroundColor = [0 114/255 189/255];
                h.lblMes.String = h.MesHarder;
                h.lblMes.BackgroundColor = h.cssCBackMesHarder;
                h.Stop.Enable = 'off';
                h.Next.Enable = 'off';
                h.Next.String = h.strNextBusy;
                h.Started = 1;
                measflag = 1;
                readflag = 1;
                set(figGUI,'UserData',h);
                % START MEASUREMENT LOOP
                PillowMeasure(figGUI);
            end
    end

    function [] = callStop(~,~)
            % Obtain user data main figure
            measflag = 0;
            readflag = 0;
            h = get(figGUI,'UserData');

            % Set status variables
            h.Terminated = 1;
            h.Completed = 0;            
            % Set controls statuses
            h.Start.Enable='off';
            h.Start.String='Terminated by user!';
            h.Start.BackgroundColor=h.cssCBackMesAbort;
            h.Stop.Enable='off';
            h.Next.Enable='on';
            h.Next.String=h.strNextSave;
            % Ask for termination reason
            h.lblMes.String = h.MesAbort;
            h.lblMes.BackgroundColor = h.cssCBackMesAbort;
            % Update user data main figure
            set(figGUI,'UserData',h);
    end

    function [] = callNext(~,~)
            % Obtain user data main figure
            measflag = 0;
            readflag = 0;
            h = get(figGUI,'UserData');
            if h.Started == 1;
                % SAVE DATA! ONLY ENABLED IF MEASUREMENT IS STARTED AND
                % COMPLETED OR TERMINATE HAS BEEN CLICKED AFTER STARTING

                %% Start saving data and tell user
                % Tell that we are saving
                h.Next.String=h.strNextSaving;
                % Calculate mean of valid forces outside pauses
                F_nop = h.DataF(h.DataP==0); % Only outside pauses
                F_val = F_nop(F_nop>=h.SmotherF); % Valid only
                %% Define data to write to files
                % writeDataOverview will be used in both Subj and All file
                writeDataOverview = {h.Subjnr,...
                    datetime('now','TimeZone','local','Format','yyyy-MM-dd'),...
                    datetime('now','TimeZone','local','Format','HH:mm:ss'),...
                    h.DateT_clock,...
                    mean(F_val,1),... % Only count valid smothering & no pause
                    max(F_nop,[],1),... % Only count no pause
                    h.DataT_run,h.DataT_smo,h.DataT_val,...
                    h.Completed,h.intPauses,h.DataNotes};

                writeDataSubj.ColHeadersOverview = celColHeadersOverview;
                writeDataSubj.DataOverview = writeDataOverview;
                writeDataSubj.ColHeadersData = celColHeadersData;
                writeDataSubj.DataSubj = [h.DataT h.DataV h.DataF h.DataP];

                %% Write experiment data to file strFileSubj
                % Existence check could be done, but for now, we just overwrite
                % the file. BE AWARE!
                save(h.strFileSubj{1}, 'writeDataSubj');
                
                % JUST AS A REFERENCE
% celColHeadersSummative= {'Nr of Subjs','SaveDate','SaveTime','Mean F_mean [N]','Mean F_max [N]',...
%                         'Mean t_run [s]','Mean t_smo [s]''Mean t_val [s]',...
%                         'Percentage Completed','Mean Pauses'};
% celColHeadersOverview = {'Subj','SaveDate','SaveTime','F_mean [N]','F_max [N]',...
%                        't_run [s]','t_smo [s]', 't_val [s]','t_smo [s]','Completed','Pauses','Notes'};
                %% Open the central file strFileAll if it exists, or make it
                if exist(strFileAll{1},'file') && ~strcmp(h.Subjnr,h.Trialstr)
                    % Add data
                    load(strFileAll{1});
                    % Overwrite headers anyway, to be sure they are correct
                    writeDataAll.ColHeadersOverview = celColHeadersOverview;
                    % Add new subject line to datalist
                    writeDataAll.DataOverview = [writeDataAll.DataOverview; writeDataOverview];
                    % Overwrite headers anyway, to be sure they are correct
                    writeDataAll.ColHeadersSummative = celColHeadersSummative;
                    % Update Summative data that sums entire dataset
                    writeDataAll.DataSummative = ...
                        {size(cell2mat(writeDataAll.DataOverview(:,1)),1),...
                        datetime('now','TimeZone','local','Format','yyyy-MM-dd'),...
                        datetime('now','TimeZone','local','Format','HH:mm:ss'),...
                        mean(cell2mat(writeDataAll.DataOverview(:,4)),1),...
                        mean(cell2mat(writeDataAll.DataOverview(:,5)),1),...
                        mean(cell2mat(writeDataAll.DataOverview(:,6)),1),...
                        max(cell2mat(writeDataAll.DataOverview(:,5)),[],1),...
                        max(cell2mat(writeDataAll.DataOverview(:,6)),[],1),...
                        mean(cell2mat(writeDataAll.DataOverview(:,7)),1),...
                        mean(cell2mat(writeDataAll.DataOverview(:,8)),1),...
                        mean(cell2mat(writeDataAll.DataOverview(:,9)),1),...
                        round(100*sum(cell2mat(writeDataAll.DataOverview(:,10))>0)/...
                        size(cell2mat(writeDataAll.DataOverview(:,10)),1)),...
                        round(mean(cell2mat(writeDataAll.DataOverview(:,11)),1),2)};
                    % Save the file
                    save(strFileAll{1},'writeDataAll');
                elseif ~exist(strFileAll{1},'file') && ~strcmp(h.Subjnr,h.Trialstr)
                    % Make headers
                    writeDataAll.ColHeadersSummative = celColHeadersSummative;
                    writeDataAll.ColHeadersOverview = celColHeadersOverview;
                    % Add first subject to datalist 
                    writeDataAll.DataOverview = writeDataOverview; 
                    % Start new summative data line on first line of
                    % DataOverview.
                    writeDataAll.DataSummative = ...
                        {size(writeDataAll.DataOverview,1),...
                        datetime('now','TimeZone','local','Format','yyyy-MM-dd'),...
                        datetime('now','TimeZone','local','Format','HH:mm:ss'),...
                        h.DateT_clock,...
                        mean(F_val,1),... % Only count valid smothering & when no pause
                        max(F_nop,[],1),... % Only count when no pause
                        mean(F_val,1),... % Only count valid smothering & when no pause
                        max(F_nop,[],1),... % Only count when no pause
                        h.DataT_run,h.DataT_smo,h.DataT_val,h.Completed,...
                        h.intPauses};

            
                    % Save the file
                    save(strFileAll{1},'writeDataAll');
                end
                % Clear writeData variables
                clear writeDataOverview writeDataSubj writeDataAll...
                writeDataSummative strFileSubj;
            end
            
            %% Reset GUI

            % Reset data variables
            h.DataT = [];
            h.DataT_run = 0;
            h.DataT_smo = 0;
            h.DataT_val = 0;
            h.DataT_clock = [];
            h.DataV = [];
            h.DataF = [];
            h.DataNotes = {};
            h.SmotherProg = '0';
            h.Subjnr = '';
            % Reset status variables
            h.Scanned = 0;
            h.Started = 0;
            h.intPauses = 0;
            h.Terminated = 0;
            h.Completed = 0;
            % Reset button properties
            h.Start.Enable='off';
            h.Start.String='Start';help addlistener
            h.Start.BackgroundColor = [126/255 211/255 0];
            h.Stop.Enable='off';
            h.Next.Enable='off';
            h.Next.String=h.strNextClean;
            % Reset edit boxes
            h.edtSubj.String='';
            h.edtNotes.String='';
            h.edtNotes.Enable='off';
            h.edtSubj.Enable='on';
            h.edtSmotherProg.String='0';
            % Reset message label
            h.lblMes.String = h.MesClear;
            h.lblMes.BackgroundColor = h.cssCBackMesClear;
            % Update user data main figure
            set(figGUI,'UserData',h);
            % Put cursor in SubjNr edit box
            uicontrol(h.edtSubj);
    end

    function [] = callCloseGUI(~,~)
        % Set run flags to zero to make PillowMeasure stop and wait two
        % sampletimes to make sure that PillowMeasure can wrap up.
        measflag = 0;
        readflag = 0;
        h = get(figGUI,'UserData');
        pause(2*h.SampleT);
    end
end
